/**********************************************************************\
**
**                         Copyright (c) 2008-2022 by
**                          HENGSTLER GmbH, Aldingen 
**
**      This  software  is furnished under a license and may be used and
**      copied only in accordance with the terms  of  such  license  and
**      with the inclusion of the above copyright notice.  This software
**      or any other copies thereof may not  be  provided  or otherwise
**      made  available  to any other person.  No title to and ownership
**      of the software is hereby transferred.
**
**      The  information  in  this software is subject to change without
**      notice and should not be construed as a commitment by Hengstler.
**
**  Module:      Xpm-Pmpd-DlgMain.cpp
**  Facility:    S690001 - Hengstler XPM Printer Driver for Windows XP/7/8/10 
**  Component:   XPM print monitor plugin demo tool (Xpm-PrintMonPluginDemo)
**  Version:     R8-V1.07-FINAL / 2022OCT18
**  Environment: Windows XP/7/8/10
**  Compiler:    Microsoft Visual Studio 2015
**  Abstract:    Handler of main dialog
**  Note:        -
**  History:     R1-V1.00c / 2008JUL11 / HOR / 1st RELEASE - 1st BETA
**               R1-V1.00d / 2008DEC11 / HOR / 1st RELEASE - 2nd BETA
**               R1-V1.00e / 2009MAY05 / HOR / 1st RELEASE - 3rd BETA
**               R1-V1.00f / 2009JUL16 / HOR / 1st RELEASE - 4th BETA
**               R1-V1.00  / 2009AUG27 / HOR / 1st RELEASE - FINAL 
**               ------------------------------------------------------------
**               R2-V1.01a / 2010JAN26 / HOR / 2nd RELEASE - 1st BETA 
**               R2-V1.01b / 2010MAR04 / HOR / 2nd RELEASE - 2nd BETA
**               R2-V1.01c / 2011DEC19 / HOR / M-R2-012-HOR
**                  Complete driver package converted to VS2010 and adapted to Windows 7 32/64 bit
**               R2-V1.01c / 2011DEC19 / HOR / 2nd RELEASE - 3rd BETA
**               R2-V1.01d / 2012JUN27 / HOR / 2nd RELEASE - 4th BETA
**               R2-V1.01e / 2015MAY05 / HOR / 2nd RELEASE - 5th BETA
**               R2-V1.01f / 2016FEB15 / HOR / 2nd RELEASE - 6th BETA
**               R2-V1.01  / 2016AUG19 / HOR / M-R2-041-HOR
**                 Complete driver package converted to VS2015 and adapted to Windows 8/10 32/64 bit
**               R2-V1.01  / 2016AUG19 / HOR / 2nd RELEASE - FINAL
**               ------------------------------------------------------------
**               R3-V1.02  / 2016AUG24 / HOR / M-R3-001-HOR
**                 Additional driver package compilation added for usage within Windows-XP.
**               R3-V1.02  / 2016AUG24 / HOR / 3rd RELEASE - FINAL
**               ------------------------------------------------------------
**               R4-V1.03  / 2018JUL10 / HOR / 4th RELEASE - FINAL
**               ------------------------------------------------------------
**               R5-V1.04  / 2018OCT11 / HOR / 5th RELEASE - FINAL
**               ------------------------------------------------------------
**               R6-V1.05  / 2018DEC07 / HOR / 6th RELEASE - FINAL
**               ------------------------------------------------------------
**               R7-V1.06  / 2022FEB10 / HOR / 7th RELEASE - FINAL
**               ------------------------------------------------------------
**               R8-V1.07  / 2022OCT18 / HOR / 8th RELEASE - FINAL
**               ------------------------------------------------------------
**
\**********************************************************************/

/*
** INCLUDES
*/

#include "stdafx.h"
#include "Shlwapi.h"
#include "winspool.h"
#include "..\..\Libraries\Xpm_PrintMonPlugin.h"
#include "..\..\Libraries\Xpm_Api.h"
#include "Xpm-Pmpd-App.h"
#include "Xpm-Pmpd-DlgMain.h"
#include ".\xpm-Pmpd-dlgmain.h"

/*
** COMPILER 
*/

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

/*
** MACROS
*/

/*
** TYPES
*/

/*
** FUNCTIONS
*/

/*
** VARIABLES
*/

/* 
** MESSAGE MAPS
*/

BEGIN_MESSAGE_MAP(CXpm_Pmpd_DlgMain, CDialog)
   ON_WM_SYSCOMMAND()
   ON_WM_PAINT()
   ON_WM_QUERYDRAGICON()
   //}}AFX_MSG_MAP
   ON_WM_TIMER()
   ON_CBN_SELCHANGE(IDC_COMBO_PRINTER_DRIVER, OnCbnSelchangeComboPrinterDriver)
   ON_BN_CLICKED(IDC_BUTTON_PRINT, OnBnClickedButtonPrint)
   ON_BN_CLICKED(IDC_BUTTON_SELECT_PRINT_FILE, OnBnClickedButtonSelectPrintFile)
END_MESSAGE_MAP()

//******************************************************************************************
//** METHOD:   CXpm_Pmpd_DlgMain::CXpm_Pmpd_DlgMain
//** ABSTRACT: Constructor
//******************************************************************************************

CXpm_Pmpd_DlgMain::CXpm_Pmpd_DlgMain
   (CWnd* pParent /*=NULL*/)
   : CDialog(CXpm_Pmpd_DlgMain::IDD, pParent)
{
   //-- Load dialog icons
   m_hIconL  =AfxGetApp()->LoadIcon(IDR_ICO_MAIN_L);
   m_hIconS = AfxGetApp()->LoadIcon(IDR_ICO_MAIN_S);

   //-- Initialize variables
   m_iTimer         = 0;
   m_pszPrinter     = NULL;
   m_fPrinting      = false;
   m_iPrintState    = 0; 
   m_iPrintFile     = 0; 
   m_dwPrintJobId   = 0;
   m_ubPrintJobId   = 0; 
   m_szPrintFile[0] = '\0';
}

//******************************************************************************************
//** METHOD:   CXpm_Pmpd_DlgMain::~CXpm_Pmpd_DlgMain
//** ABSTRACT: Destructor
//******************************************************************************************

CXpm_Pmpd_DlgMain::~CXpm_Pmpd_DlgMain
   ()
{
   //-- Free printer if allocated
   if (m_pszPrinter) free(m_pszPrinter); m_pszPrinter= NULL;
}

//******************************************************************************************
//** METHOD:   CXpm_Pmpd_DlgMain::DoDataExchange
//** ABSTRACT: This method is called by the framework to exchange and validate dialog data.
//******************************************************************************************

void CXpm_Pmpd_DlgMain::DoDataExchange
   (CDataExchange* pDX)
{
   //-- ...
   CDialog::DoDataExchange(pDX);
}

//******************************************************************************************
//** METHOD:   CXpm_Pmpd_DlgMain::OnInitDialog
//** ABSTRACT: This method is called by the framework in response to the WM_INITDIALOG message
//******************************************************************************************

BOOL CXpm_Pmpd_DlgMain::OnInitDialog
   ()
{
   //-- Init dialog
   CDialog::OnInitDialog();

   //-- Set dialog icons
   SetIcon(m_hIconL, TRUE);   
   SetIcon(m_hIconS, FALSE);  

   //-- Modify system menu and add dialogs
   CMenu* pSysMenu = this->GetSystemMenu(FALSE);

   //-- Setup combo box: printer driver
   { 
      CComboBox * poComboBox;   // Pointer to combo box
 
      //-- Get pointer to combo box
      if ((poComboBox = ((CComboBox *) this->GetDlgItem(IDC_COMBO_PRINTER_DRIVER))) != NULL) {

         HANDLE hPrinterEnum;  // Handle of printer enumeration

         //-- Reset contents of box
         poComboBox->ResetContent() ;

         //-- Start printer enumeration
         if ((hPrinterEnum = Xpm_Api_Pe_PrinterEnumStart()) != NULL) {

            PSTR pszPrinterName;  // Name of printer

            //-- Go through all printers
            for (pszPrinterName = Xpm_Api_Pe_PrinterEnum(hPrinterEnum, true); pszPrinterName; pszPrinterName = Xpm_Api_Pe_PrinterEnum(hPrinterEnum, false)) {
 
                //-- Add printer
                poComboBox->AddString(pszPrinterName);
            }

            //-- End of printer enumeration
            Xpm_Api_Pe_PrinterEnumEnd(hPrinterEnum);
         }
 
         //-- Select printer
         poComboBox->SetCurSel(0);
      }
   }
   
   //-- Get current driver name 
   OnCbnSelchangeComboPrinterDriver();

   //-- Update dialog
   UpdateDialog();

   //-- Setup update timer to .. hz
   m_iTimer = SetTimer(1, 100, NULL);

   //-- All done
   return TRUE; 
}

//******************************************************************************************
//** METHOD:   CXpm_Pmpd_DlgMain::OnSysCommand
//** ABSTRACT: This method is called by the framework when the user selects a command from 
//**          the Control menu, or when the user selects the Maximize or the Minimize button.
//******************************************************************************************

void CXpm_Pmpd_DlgMain::OnSysCommand
   (UINT nID, LPARAM lParam)
{
    CDialog::OnSysCommand(nID, lParam);
}

//******************************************************************************************
//** METHOD:   CXpm_Pmpd_DlgMain::OnPaint
//** ABSTRACT: This method is called by the framework when an application makes a request to 
//**          repaint a portion of an application window. 
//******************************************************************************************

void CXpm_Pmpd_DlgMain::OnPaint
   () 
{
   if (IsIconic()) {

      CPaintDC dc(this); //--Gertekontext zum Zeichnen

      SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

      //-- Symbol in Clientrechteck zentrieren
      int cxIcon = GetSystemMetrics(SM_CXICON);
      int cyIcon = GetSystemMetrics(SM_CYICON);
      CRect rect;
      GetClientRect(&rect);
      int x = (rect.Width() - cxIcon + 1) / 2;
      int y = (rect.Height() - cyIcon + 1) / 2;

      //-- Symbol zeichnen
      dc.DrawIcon(x, y, this->m_hIconS);
   }
   else {
      CDialog::OnPaint();
   }
}

//******************************************************************************************
//** METHOD:   CXpm_Pmpd_DlgMain::OnQueryDragIcon
//** ABSTRACT: The framework calls this member function by a minimized (iconic) window that 
//**          does not have an icon defined for its class
//******************************************************************************************

HCURSOR CXpm_Pmpd_DlgMain::OnQueryDragIcon
   ()
{
   return static_cast<HCURSOR>(this->m_hIconS);
}

//******************************************************************************************
//** METHOD     CXpm_Pmpd_DlgMain::OnTimer
//** ABSTRACT   Cyclic timer event (1 sec ) to update structures from registry and 
//             dialog elements
//******************************************************************************************

void CXpm_Pmpd_DlgMain::OnTimer
   (UINT nIDEvent)
{
   //-- Update dialog
   UpdateDialog();
}

//******************************************************************************************
//** METHOD     CXpm_Pmpd_DlgMain::OnCbnSelchangeComboPrinterDriver
//** ABSTRACT   Handles printer driver selection combo box
//******************************************************************************************

void CXpm_Pmpd_DlgMain::OnCbnSelchangeComboPrinterDriver
   ()
{
   //-- Free printer if allocated
   if (m_pszPrinter) free(m_pszPrinter); m_pszPrinter= NULL;

   //-- Get combo box selection: printer driver
   { 
      CComboBox * poComboBox;   // Pointer to combo box
      INT         iSel;         // Selected item
 
      //-- Get pointer to combo box
      if ((poComboBox = ((CComboBox *) this->GetDlgItem(IDC_COMBO_PRINTER_DRIVER))) != NULL) {

         //-- Get selected item
         if ((iSel = poComboBox->GetCurSel()) != CB_ERR) {

            //-- Allocate memory for printer name 
            if ((m_pszPrinter = (PSTR) malloc(poComboBox->GetLBTextLen(iSel)+1)) != NULL) {

               //-- Get text of selected item
               poComboBox->GetLBText(iSel, m_pszPrinter);
            }
         }
      }
   }

   //-- Update dialog
   UpdateDialog();
}

//******************************************************************************************
//** METHOD     CXpm_Pmpd_DlgMain::ReadPrinterDataFromRegistry
//** ABSTRACT   Reads printer data from registry
//******************************************************************************************

bool CXpm_Pmpd_DlgMain::ReadPrinterDataFromRegistry
   (PSTR pszPrinter, PSTR pszKey, PSTR pszValue, PBYTE pubData, DWORD dwSize)
{
   bool                            fResult                    = false;  // Return value, FALSE by default
   PRINTER_DEFAULTS                sPrinterDefaults;                    // Printer defaults structure
   HANDLE                          hPrinter                   = NULL;   // Handle of printer

   //-- Validate parameter
   if (!pszPrinter || !pszKey || !pszValue || !pubData || !dwSize)
      goto FAILURE;

   //-- Setup printer defaults
   ZeroMemory(&sPrinterDefaults, sizeof(sPrinterDefaults));
   sPrinterDefaults.DesiredAccess = PRINTER_ALL_ACCESS;
  
   //-- Open printer 
   if (!OpenPrinter(pszPrinter, &hPrinter, &sPrinterDefaults)) 
      goto FAILURE;

   //-- Read data from registery and reset data if not (yet) available or if revision nr differs
   {
      DWORD dwType;   //--Type of registry data
      DWORD dwNeeded; //--Bytes needed
   
       if (    (GetPrinterDataExA(hPrinter, pszKey, pszValue, &dwType, pubData, dwSize, &dwNeeded)  != ERROR_SUCCESS) 
            || (dwType != REG_BINARY)
          ) {
            goto FAILURE;
      }
   }

   //-- Close printer
   ClosePrinter(hPrinter); hPrinter = NULL;

   //-- Success
   fResult = true;

   //-- All done
   goto EXIT;

FAILURE:
   ;

EXIT:

   //-- Close printer if open
   if (hPrinter) ClosePrinter(hPrinter); hPrinter = NULL;

   //-- Return TRUE on success and FALSE otherwise
   return(fResult);
}

//************************************************************************
//** METHOD   CXpm_Pmpd_DlgMain::UpdateJobInfoInDialog
//** ABSTRACT Updates job information in the main dialog
//************************************************************************

void CXpm_Pmpd_DlgMain::UpdateJobInfoInDialog
   (PS_XPM_PMP_JOB_INFO psJobInfo)
{
   DWORD                dwAgeSec   = -1;        // Age of job information in sec
   CString              strItem;                // A string

   //-- Update registry path to job information
   {
      //-- Path is ...\KEY\VALUE
      strItem += "...\\";
      strItem += D_XPM_PMP_REG_KEY;
      strItem += "\\";
      strItem += D_XPM_PMP_REG_VALUE_PMP_JOB_INFO;
      GetDlgItem(IDC_TXT_JI_REGISTRY_ENTRY)->SetWindowText(strItem);
   }

   //-- Determine age of job information if valid
   if (psJobInfo && (psJobInfo->dwTsLastUpdate != (DWORD) -1)) 
        dwAgeSec = (GetTickCount() - psJobInfo->dwTsLastUpdate) / 1000;
   else dwAgeSec = (DWORD) -1; 

   //-- Update age of job information
   {
      //-- Regard update timestamp valid if ...
      //..     job info is valid 
      //.. AND age differs from 0xFFFFFFFF
      if (psJobInfo && (dwAgeSec != (DWORD) -1)) 
           strItem.Format("%u sec", dwAgeSec);
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_JI_TIME_SINCE_LAST_UPDATE)->SetWindowText(strItem);
   }

   //-- Update job identifier
   { 
      //-- Regard job identifier valid if ...
      //..     job info is valid 
      //.. AND age less than ... ms 
      if (psJobInfo && (dwAgeSec < 99999)) {
         if (psJobInfo->dwJobId != (DWORD) -1)
              strItem.Format("%d", psJobInfo->dwJobId);
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_JI_JOB_IDENTIFIER)->SetWindowText(strItem);
   }

   //-- Update job status
   { 
      //-- Regard job status valid if ...
      //..     job info is valid 
      //.. AND age less than ... ms 
      if (psJobInfo && (dwAgeSec < 99999)) {
         if (psJobInfo->dwJobId != (DWORD) -1)
              strItem.Format("%d", psJobInfo->dwJobStatus);
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_JI_JOB_STATUS)->SetWindowText(strItem);
   }

   //-- Update page count 
   {
      //-- Regard page count valid if ...
      //..     job info is valid 
      //.. AND age less than ... ms 
      if (psJobInfo && (dwAgeSec < 99999)) {
         if (psJobInfo->dwJobId != (DWORD) -1)
              strItem.Format("%d", psJobInfo->dwPageCount);
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_JI_PAGE_COUNT)->SetWindowText(strItem);
   }
}

//************************************************************************
//** METHOD   CXpm_Pmpd_DlgMain::UpdatePrinterInfoInDialog
//** ABSTRACT Updates printer information in the main dialog
//************************************************************************

void CXpm_Pmpd_DlgMain::UpdatePrinterInfoInDialog
   (PS_XPM_PMP_PRINTER_INFO psPrinterInfo)
{
   DWORD                dwAgeSec   = -1;        // Age of printer information in seconds
   CString              strItem;                // A string

   //-- Update registry path to printer information
   {
      //-- Path is ...\KEY\VALUE
      strItem += "...\\";
      strItem += D_XPM_PMP_REG_KEY;
      strItem += "\\";
      strItem += D_XPM_PMP_REG_VALUE_PMP_PRINTER_INFO;
      GetDlgItem(IDC_TXT_PI_REGISTRY_ENTRY)->SetWindowText(strItem);
   }

   //-- Determine age of printer information if valid 
   if (psPrinterInfo && (psPrinterInfo->dwTsLastUpdate != (DWORD) -1)) 
        dwAgeSec = (GetTickCount() - psPrinterInfo->dwTsLastUpdate) / 1000;
   else dwAgeSec = (DWORD) -1; 

   //-- Update age of printer information
   {
      //-- Regard update timestamp valid if ...
      //..     printer info is valid 
      //.. AND age differs from 0xFFFFFFFF
      if (psPrinterInfo && (dwAgeSec != (DWORD) -1))
           strItem.Format("%u sec", dwAgeSec);
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_PI_TIME_SINCE_LAST_UPDATE)->SetWindowText(strItem);
   }

   //-- Update communication protocol revision
   { 
      //-- Regard communication protocol revision valid if ...
      //..     printer info is valid 
      //.. AND age less than ... ms 
      if (psPrinterInfo && (dwAgeSec < 99999)) {
         if (psPrinterInfo->dwDataValidFlag & D_XPM_PMP_PIN_DVF_PRINTER_INFO) 
              strItem.Format("%d", psPrinterInfo->sPrinterInfo.ubProtocolRevision);
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_PI_COMMUNICATION_PROTOCOL_REVISION)->SetWindowText(strItem);
   }

   //-- Update printer part number 
   { 
      //-- Regard printer part number valid if ...
      //..     printer info is valid 
      //.. AND age less than ... ms 
      if (psPrinterInfo && (dwAgeSec < 99999)) {
         if (    (psPrinterInfo->dwDataValidFlag & D_XPM_PMP_PIN_DVF_PRINTER_INFO) 
              && psPrinterInfo->sPrinterInfo.szPartNr[0]
            ) strItem = psPrinterInfo->sPrinterInfo.szPartNr;
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_PI_PART_NUMBER)->SetWindowText(strItem);
   }

   //-- Update printer serial number 
   { 
      //-- Regard printer serial number valid if ...
      //..     printer info is valid 
      //.. AND age less than ... ms 
      if (psPrinterInfo && (dwAgeSec < 99999)) {
         if (    (psPrinterInfo->dwDataValidFlag & D_XPM_PMP_PIN_DVF_PRINTER_INFO) 
              && psPrinterInfo->sPrinterInfo.szSerialNr[0]
            ) strItem = psPrinterInfo->sPrinterInfo.szSerialNr;
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_PI_SERIAL_NUMBER)->SetWindowText(strItem);
   }

   //-- Update date of manufacturing 
   { 
      //-- Regard date of manufacturing valid if ...
      //..     printer info is valid 
      //.. AND age less than ... ms 
      if (psPrinterInfo && (dwAgeSec < 99999)) {
         if (    (psPrinterInfo->dwDataValidFlag & D_XPM_PMP_PIN_DVF_PRINTER_INFO) 
              && psPrinterInfo->sPrinterInfo.szDateOfManuf[0]
            ) strItem = psPrinterInfo->sPrinterInfo.szDateOfManuf;
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_PI_DATE_OF_MANUFACTURING)->SetWindowText(strItem);
   }

   //-- Update model code
   { 
      //-- Regard model code valid if ...
      //..     printer info is valid 
      //.. AND age less than ... ms 
      if (psPrinterInfo && (dwAgeSec < 99999)) {
         if (psPrinterInfo->dwDataValidFlag & D_XPM_PMP_PIN_DVF_PRINTER_INFO) 
              strItem.Format("[%8.8x]", psPrinterInfo->sPrinterInfo.dwModelCode);
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_PI_MODEL_CODE)->SetWindowText(strItem);
   }

   //-- Update configuration code
   { 
      //-- Regard configuration code valid if ...
      //..     printer info is valid 
      //.. AND age less than ... ms 
      if (psPrinterInfo && (dwAgeSec < 99999)) {
         if (psPrinterInfo->dwDataValidFlag & D_XPM_PMP_PIN_DVF_PRINTER_INFO) 
              strItem.Format("[%8.8x]", psPrinterInfo->sPrinterInfo.dwConfigCode);
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_PI_CONFIGURATION_CODE)->SetWindowText(strItem);
   }

   //-- Update sensor code
   { 
      //-- Regard sensor code valid if ...
      //..     printer info is valid 
      //.. AND age less than ... ms 
      if (psPrinterInfo && (dwAgeSec < 99999)) {
         if (psPrinterInfo->dwDataValidFlag & D_XPM_PMP_PIN_DVF_PRINTER_INFO) 
              strItem.Format("[%8.8x]", psPrinterInfo->sPrinterInfo.dwSensorCode);
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_PI_SENSOR_CODE)->SetWindowText(strItem);
   }

   //-- Update feature code
   { 
      //-- Regard feature code valid if ...
      //..     printer info is valid 
      //.. AND age less than ... ms 
      if (psPrinterInfo && (dwAgeSec < 99999)) {
         if (psPrinterInfo->dwDataValidFlag & D_XPM_PMP_PIN_DVF_PRINTER_INFO) 
              strItem.Format("[%8.8x]", psPrinterInfo->sPrinterInfo.dwFeatureCode);
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_PI_FEATURE_CODE)->SetWindowText(strItem);
   }

   //-- Update firmware part number
   { 
      //-- Regard firmware part number valid if ...
      //..     printer info is valid 
      //.. AND age less than ... ms 
      if (psPrinterInfo && (dwAgeSec < 99999)) {
         if (psPrinterInfo->dwDataValidFlag & D_XPM_PMP_PIN_DVF_PRINTER_INFO) 
              strItem = psPrinterInfo->sPrinterInfo.szFwPartNr;
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_PI_FIRMWARE_PART_NR)->SetWindowText(strItem);
   }

   //-- Update firmware version
   { 
      //-- Regard firmware version valid if ...
      //..     printer info is valid 
      //.. AND age less than ... ms 
      if (psPrinterInfo && (dwAgeSec < 99999)) {
         if (psPrinterInfo->dwDataValidFlag & D_XPM_PMP_PIN_DVF_PRINTER_INFO) 
              strItem = psPrinterInfo->sPrinterInfo.szFwVersion;
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_PI_FIRMWARE_VERSION)->SetWindowText(strItem);
   }

   //-- Update firmware date
   { 
      //-- Regard firmware date valid if ...
      //..     printer info is valid 
      //.. AND age less than ... ms 
      if (psPrinterInfo && (dwAgeSec < 99999)) {
         if (psPrinterInfo->dwDataValidFlag & D_XPM_PMP_PIN_DVF_PRINTER_INFO) 
              strItem = psPrinterInfo->sPrinterInfo.szFwDate;
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_PI_FIRMWARE_DATE)->SetWindowText(strItem);
   }
}

//************************************************************************
//** METHOD   CXpm_Pmpd_DlgMain::UpdatePrinterStatusInDialog
//** ABSTRACT Updates status information in the main dialog
//************************************************************************

void CXpm_Pmpd_DlgMain::UpdatePrinterStatusInDialog
   (PS_XPM_PMP_PRINTER_STATUS psPrinterStatus)
{
   DWORD                dwAgeSec   = -1;        // Age of status information in seconds
   CString              strItem;                // A string

   //-- Update registry path to printer information
   {
      //-- Path is ...\KEY\VALUE
      strItem += "...\\";
      strItem += D_XPM_PMP_REG_KEY;
      strItem += "\\";
      strItem += D_XPM_PMP_REG_VALUE_PMP_PRINTER_STATUS;
      GetDlgItem(IDC_TXT_SI_REGISTRY_ENTRY)->SetWindowText(strItem);
   }

   //-- Determine age of status information if valid 
   if (psPrinterStatus && (psPrinterStatus->dwTsLastUpdate != (DWORD) -1)) 
        dwAgeSec = (GetTickCount() - psPrinterStatus->dwTsLastUpdate) / 1000;
   else dwAgeSec = (DWORD) -1; 

   //-- Update age of printer information
   {
      //-- Regard update timestamp valid if ...
      //..     printer info is valid 
      //.. AND age differs from 0xFFFFFFFF
      if (psPrinterStatus && (dwAgeSec != (DWORD) -1))
           strItem.Format("%u sec", dwAgeSec);
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_SI_TIME_SINCE_LAST_UPDATE)->SetWindowText(strItem);
   }

   //-- Update status bits
   { 
      //-- Regard status bits valid if ...
      //..     printer status is valid 
      //.. AND age less than ... ms 
      if (psPrinterStatus && (dwAgeSec < 99999)) {
         if (psPrinterStatus->dwDataValidFlag & D_XPM_PMP_PST_DVF_PRINTER_STATUS) 
              strItem.Format("[%8.8x]", psPrinterStatus->sPrinterStatus.dwStatus);
         else strItem.Format("<UNDEFINED>");
      } 
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_SI_STATUS_BITS)->SetWindowText(strItem);
   }

   //-- Update job identifier
   { 
      //-- Regard job identifier valid if ...
      //..     printer status is valid  
      //.. AND age less than ... ms 
      if (psPrinterStatus && (dwAgeSec < 99999)) {
         if (psPrinterStatus->dwDataValidFlag & D_XPM_PMP_PST_DVF_PRINTER_STATUS) 
              strItem.Format("%d", (WORD) psPrinterStatus->sPrinterStatus.ubPrintJobId);
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_SI_JOB_IDENTIFIER)->SetWindowText(strItem);
   }

   //-- Update print head temperature
   { 
      //-- Regard print head temperature valid if ...
      //..     printer status is valid  
      //.. AND age less than ... ms 
      if (psPrinterStatus && (dwAgeSec < 99999)) {
         if (psPrinterStatus->dwDataValidFlag & D_XPM_PMP_PST_DVF_PRINTER_STATUS) 
              strItem.Format("%+d C", (INT) psPrinterStatus->sPrinterStatus.siPrintHeadTempC);
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_SI_PH_TEMPERATURE)->SetWindowText(strItem);
   }

   //-- Update board voltage
   { 
      //-- Regard board voltage valid if ...
      //..     printer status is valid  
      //.. AND age less than ... ms 
      if (psPrinterStatus && (dwAgeSec < 99999)) {
         if (psPrinterStatus->dwDataValidFlag & D_XPM_PMP_PST_DVF_PRINTER_STATUS) 
              strItem.Format("%2.1lf V", (double) psPrinterStatus->sPrinterStatus.wBoardVoltageTv / 10);
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_SI_BOARD_VOLTAGE)->SetWindowText(strItem);
   }

   //-- Update printer state
   { 
      //-- Regard printer state valid if ...
      //..     printer status is valid 
      //.. AND age less than ... ms 
      if (psPrinterStatus && (dwAgeSec < 99999)) {
         if (psPrinterStatus->dwDataValidFlag & D_XPM_PMP_PST_DVF_PRINTER_STATUS) 
              strItem.Format("%d", (WORD) psPrinterStatus->sPrinterStatus.ubState);
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_SI_PRINTER_STATE)->SetWindowText(strItem);
   }

   //-- Update printer error
   { 
      //-- Regard printer error if ...
      //..     printer status is valid 
      //.. AND age less than ... ms 
      if (psPrinterStatus && (dwAgeSec < 99999)) {
         if (psPrinterStatus->dwDataValidFlag & D_XPM_PMP_PST_DVF_PRINTER_STATUS) 
              strItem.Format("%d", (WORD) psPrinterStatus->sPrinterStatus.wErrorCode);
         else strItem.Format("<UNDEFINED>");
      }
      else strItem = "<INVALID>";
      GetDlgItem(IDC_TXT_SI_PRINTER_ERROR)->SetWindowText(strItem);
   }
}

//******************************************************************************************
//** METHOD    CXpm_Pmpd_DlgMain::UpdateDialog
//** ABSTRACT  Retrieves registry data supplied by print monitor plugins and updates the 
//**           corresponding dialog items
//******************************************************************************************

void CXpm_Pmpd_DlgMain::UpdateDialog
   ()
{
   //-- Read job information from registry and update dialog 
   { 
      TS_XPM_PMP_JOB_INFO sJobInfo;  // Job information structure as stored in registry

      //-- Read job info from registry and check revision
      if (     ReadPrinterDataFromRegistry(m_pszPrinter, D_XPM_PMP_REG_KEY, D_XPM_PMP_REG_VALUE_PMP_JOB_INFO, (PBYTE) &sJobInfo, sizeof(sJobInfo))
           && (sJobInfo.ubRevision == D_XPM_PMP_JI_REVISION)
         ) UpdateJobInfoInDialog(&sJobInfo);
      else UpdateJobInfoInDialog(NULL);
   }

   //-- Read printer information from registry and update dialog
   { 
      TS_XPM_PMP_PRINTER_INFO sPrinterInfo; // Printer information structure as stored in registry

      //-- Read printer info from registry and check revision
      if (     ReadPrinterDataFromRegistry(m_pszPrinter, D_XPM_PMP_REG_KEY, D_XPM_PMP_REG_VALUE_PMP_PRINTER_INFO, (PBYTE) &sPrinterInfo, sizeof(sPrinterInfo))
           && (sPrinterInfo.ubRevision == D_XPM_PMP_PIN_REVISION)
         ) UpdatePrinterInfoInDialog(&sPrinterInfo);
      else UpdatePrinterInfoInDialog(NULL);
   }

   //-- Read status information from registry and update dialog
   { 
      TS_XPM_PMP_PRINTER_STATUS sPrinterStatus; // Status information structure as stored in registry

      //-- Read printer info from registry and check revision
      if (     ReadPrinterDataFromRegistry(m_pszPrinter, D_XPM_PMP_REG_KEY, D_XPM_PMP_REG_VALUE_PMP_PRINTER_STATUS, (PBYTE) &sPrinterStatus, sizeof(sPrinterStatus))
           && (sPrinterStatus.ubRevision == D_XPM_PMP_PST_REVISION)
         ) UpdatePrinterStatusInDialog(&sPrinterStatus);
      else UpdatePrinterStatusInDialog(NULL);
   }


   //-- Generate printouts 
   {
      TS_XPM_PMP_JOB_INFO       sJobInfo;                       // Job information structure as stored in registry
      TS_XPM_PMP_PRINTER_STATUS sPrinterStatus;                 // Status information structure as stored in registry
      CString                   strTxt;                         // Some text string
      PRINTER_DEFAULTSA         sPrinterDefaults;               // Printer defaults structure
      HANDLE                    hPrinter          = NULL;       // Handle of printer

      //-- Setup printer defaults
      ZeroMemory(&sPrinterDefaults, sizeof(sPrinterDefaults));
      sPrinterDefaults.DesiredAccess = PRINTER_ALL_ACCESS;

      //-- Check if printing is disabled
      if (!m_fPrinting) {
    
          //-- Show appropriate message and reset state
          GetDlgItem(IDC_TXT_PRINT_STATUS)->SetWindowText("PRINTING DISABLED");
          m_iPrintState = 0;
          m_iPrintFile  = 0; 
      }
      else
      //.. Get current job info and printer status 
      if (    !ReadPrinterDataFromRegistry(m_pszPrinter, D_XPM_PMP_REG_KEY, D_XPM_PMP_REG_VALUE_PMP_JOB_INFO, (PBYTE) &sJobInfo, sizeof(sJobInfo))
           || (sJobInfo.ubRevision != D_XPM_PMP_JI_REVISION)
           || !ReadPrinterDataFromRegistry(m_pszPrinter, D_XPM_PMP_REG_KEY, D_XPM_PMP_REG_VALUE_PMP_PRINTER_STATUS, (PBYTE) &sPrinterStatus, sizeof(sPrinterStatus))
           || (sPrinterStatus.ubRevision != D_XPM_PMP_PST_REVISION)
         ) {
 
         //-- Show appropriate message   
         GetDlgItem(IDC_TXT_PRINT_STATUS)->SetWindowText("COULD NOT READ JOB INFO AND PRINTER STATUS FROM REGISTRY");
         m_iPrintState = 0;
      }
      else
      //.. Check age of printer status being more than ... ms
      if (GetTickCount() - sPrinterStatus.dwTsLastUpdate > 60000) {

         //-- Show appropriate message   
         GetDlgItem(IDC_TXT_PRINT_STATUS)->SetWindowText("PRINTER DISCONNECTED");
         m_iPrintState = 0;
      }
      else 
      //.. Open connection to printer
      if (!OpenPrinter(m_pszPrinter, &hPrinter, &sPrinterDefaults)) {

          //-- Show appropriate message and reset state
          GetDlgItem(IDC_TXT_PRINT_STATUS)->SetWindowText("COULD NOT OPEN PRINTER");
          m_iPrintState = 0;
      }
      else { 

        int iPrintStatePrev = m_iPrintState;

         //-- Handle according to print status 
         switch (m_iPrintState) {

            case 0: //-- Verify if printing is possible
                 {
                    DWORD  dwBytesJobInfo=1; // Number of bytes of storage required for enum jobs
                    DWORD  dwDummy;          // Dummy double word

                    //-- Show appropriate message   
                    GetDlgItem(IDC_TXT_PRINT_STATUS)->SetWindowText("CHECKING PRINTER/SPOOLER");

                    //-- Determine number of print jobs in the spooler 
                    EnumJobs(hPrinter, 0, 1, 1, (LPBYTE) NULL, dwBytesJobInfo, (LPDWORD)&dwBytesJobInfo, &dwDummy);

                    //-- Check if spooler is empty
                    if (dwBytesJobInfo != 0) {
 
                       //-- Show appropriate message and goto corresponding state
                       GetDlgItem(IDC_TXT_PRINT_STATUS)->SetWindowText("SPOOLER IS NOT EMPTY");
                    }
                    else 
                    //.. Check if the printer driver is not printing
                    if (sJobInfo.dwJobStatus == 1) {

                       //-- Show appropriate message   
                       GetDlgItem(IDC_TXT_PRINT_STATUS)->SetWindowText("SPOOLER IS PRINTING");
                    }
                    else
                    //.. Check if the printer is operational
                    if ((sPrinterStatus.sPrinterStatus.dwStatus & D_XPM_API_PSDW_PRINTER_NOT_READY) != 0) {

                       //-- Show appropriate message   
                       GetDlgItem(IDC_TXT_PRINT_STATUS)->SetWindowText("PRINTER IS NOT READY");
                    } 
                    else {

                       //-- Printer ready for printing => goto next state 
                       m_iPrintState = 1;
                    }
                 }

                 break;

            case 1: //-- Send print job trough spooler
                 {
                    DWORD        dwBinary   = 0;       // Size of binary file
                    FILE       * pfBinary   = NULL;    // File descriptor
                    BYTE       * pubBinary  = NULL;    // Pointer to binary (from file)

                    //-- Show appropriate message   
                    strTxt.Format("PRINTING DOCUMENT #%d", m_iPrintFile);
                    GetDlgItem(IDC_TXT_PRINT_STATUS)->SetWindowText(strTxt);

                    //-- Reset print job id
                    m_dwPrintJobId = 0;

                    //-- Open print file 
                    if (     !PathFileExists(m_szPrintFile)
                          || ((pfBinary = fopen(m_szPrintFile, "rb")) == NULL) 
                        ) {

                       //-- Show appropriate message   
                       GetDlgItem(IDC_TXT_PRINT_STATUS)->SetWindowText("OPENING PRINT FILE FAILED");
                    }
                    else 
                    //.. Determine size of print file
                    if (    (fseek(pfBinary, 0, SEEK_END) != 0) 
                         || ((dwBinary = ftell(pfBinary)) == 0) 
                         || (fseek(pfBinary, 0, SEEK_SET) != 0)
                       ) {

                       //-- Show appropriate message   
                       GetDlgItem(IDC_TXT_PRINT_STATUS)->SetWindowText("PRINT FILE SIZE MISMATCH");
                    }
                    else 
                    //.. Allocate memory for print data
                    if ((pubBinary = (PBYTE) malloc(dwBinary+1)) == NULL) {

                       //-- Show appropriate message   
                       GetDlgItem(IDC_TXT_PRINT_STATUS)->SetWindowText("COULD NOT ALLOCATE MEMORY FOR PRINT DATA");
                    }
                    else 
                    //.. Read data from print file
                    if (fread(pubBinary, 1, dwBinary, pfBinary) != dwBinary) {

                       //-- Show appropriate message   
                       GetDlgItem(IDC_TXT_PRINT_STATUS)->SetWindowText("COULD NOT READ DATA FROM PRINT FILE");
                    }
                    else {
  
                       DOC_INFO_1   sDocInfo;             // Document info for StartDocPrinter

                       //-- Setup document info
                       sDocInfo.pDocName    = "XPM PRINT MON PLUGIN DEMO";
                       sDocInfo.pOutputFile = NULL; 
                       sDocInfo.pDatatype   = "RAW"; 

                       //-- Start of printout 
                       if ((m_dwPrintJobId = StartDocPrinter(hPrinter, 1, (LPBYTE) &sDocInfo)) == 0) {
   
                          //-- Show appropriate message   
                          GetDlgItem(IDC_TXT_PRINT_STATUS)->SetWindowText("COULD NOT PRINT DOCUMENT");
                       }
                       else {

                          BYTE aubCmdParameter[] = {0x1C,0x72,0x00}; // Parameter command at end of print job
                          DWORD dwWritten;

                          //-- Write data to printer
                          WritePrinter(hPrinter, (LPSTR) pubBinary, dwBinary, &dwWritten);
   
                          //-- Setup and write printer status parameter
                          m_ubPrintJobId = max((BYTE) (sPrinterStatus.sPrinterStatus.ubPrintJobId + 2), 1);
                          aubCmdParameter[2] = m_ubPrintJobId;
                          WritePrinter(hPrinter, (LPSTR) aubCmdParameter, sizeof(aubCmdParameter), &dwWritten);

                           //-- End of printout
                          EndDocPrinter(hPrinter);
  
                          //-- Increase print file
                          m_iPrintFile++; 
                       }
                    }

                    //-- Check if a print has been send
                    if (m_dwPrintJobId > 0) {

                       //-- Printout send to printer => goto next state 
                       m_iPrintState = 2;
                    }
                    else {
                    
                       //-- Printing failed => goto next state 
                       m_iPrintState = 4;
                    }

                    //-- Close binary file if opened 
                    if (pfBinary != NULL)  {
                       fclose(pfBinary);
                       pfBinary = NULL;
                    }

                    //-- Free memory allocated for binary data
                    if (pubBinary) {
                       free(pubBinary);
                       pubBinary = NULL;
                    }
                 }
                 break;
                 
            case 2: //-- Wait until the printing by the print monitor is finished
                 {
                    //-- Show appropriate message   
                    strTxt.Format("WAITING FOR PRINT MONITOR TO FINISH PRINTING DOCUMENT #%d", m_iPrintFile);
                    GetDlgItem(IDC_TXT_PRINT_STATUS)->SetWindowText(strTxt);

                    //-- Check if the spooler has finished the print job 
                    if (    (sJobInfo.dwJobId == m_dwPrintJobId) 
                         && (sJobInfo.dwJobStatus == D_XPM_PMP_JI_JS_JOB_FINISHED) 
                       ) {
 
                          //-- Spooler has sent print data to the printer => goto next state 
                          m_iPrintState = 3;
                       }
                    else
                    //-- Check if the spooler has canceled the print job 
                    if (    (sJobInfo.dwJobId == m_dwPrintJobId) 
                         && (sJobInfo.dwJobStatus == D_XPM_PMP_JI_JS_JOB_CANCELLED) 
                       ) {

                       //-- Show appropriate message   
                       GetDlgItem(IDC_TXT_PRINT_STATUS)->SetWindowText("SPOOLER CANCELLED PRINT");
                       m_iPrintState = 4;
                    }
                    else 
                    //-- Check for a time out 
                    if ((GetTickCount() - m_tsPrintStateMs) > 10000) {

                       //-- Show appropriate message   
                       GetDlgItem(IDC_TXT_PRINT_STATUS)->SetWindowText("SPOOLER DID NOT FINISH PRINTING");
                       m_iPrintState = 4;
                    }
                 }
                 break;
                               
            case 3: //-- Wait until the printing by the printer is completed
                 {
                    //-- Show appropriate message   
                    strTxt.Format("WAITING FOR PRINTER TO FINISH PRINTING DOCUMENT #%d", m_iPrintFile);
                    GetDlgItem(IDC_TXT_PRINT_STATUS)->SetWindowText(strTxt);

                    //-- Check if the printer is operational
                    if ((sPrinterStatus.sPrinterStatus.dwStatus & D_XPM_API_PSDW_PRINTER_NOT_READY) != 0) {

                       //-- Printer has failed printing the file => goto next state 
                       GetDlgItem(IDC_TXT_PRINT_STATUS)->SetWindowText("PRINTER REPORTED PRINT FAILURE");
                       m_iPrintState = 4;
                    } 
                    else
                    //-- Check if the printer has printed the print job 
                    if (sPrinterStatus.sPrinterStatus.ubPrintJobId == m_ubPrintJobId) {

                       //-- Printer has printed the file => goto next state 
                       m_iPrintState = 0;
                    }
                    else 
                    //-- Check for a time out 
                    if ((GetTickCount() - m_tsPrintStateMs) > 10000) {

                       //-- Show appropriate message   
                       GetDlgItem(IDC_TXT_PRINT_STATUS)->SetWindowText("PRINTER DID NOT FINISH PRINTING");
                       m_iPrintState = 4;
                    }
                 }
                 break;

            case 4: //-- Wait .. sec 
                   
                 //-- Check if delay has ellapsed
                 if ((GetTickCount() - m_tsPrintStateMs) > 1000) {

                    //-- Retry to print => goto next state 
                    m_iPrintState = 0;
                 }

                 break;
         }

         //-- Check for a new state
         if (m_iPrintState != iPrintStatePrev) {

            //-- Reset timestamp
            m_tsPrintStateMs = GetTickCount();
         }

         //-- Close printer
         ClosePrinter(hPrinter); hPrinter = NULL;
      }
   }
}

//******************************************************************************************
// METHOD:   CXpm_DitDlgMain::OnBnClickedButtonPrint
// ABSTRACT: Handler for select print enable/disable button being pressed
//******************************************************************************************

void CXpm_Pmpd_DlgMain::OnBnClickedButtonPrint()
{
   //-- Enable/disable printing 
   m_fPrinting = !m_fPrinting;

   //-- Update dialog text 
   GetDlgItem(IDC_BUTTON_PRINT)->SetWindowText(m_fPrinting ? "STOP PRINTING" : "START PRINTING");
}


//******************************************************************************************
// METHOD:   CXpm_DitDlgMain::OnBnClickedButtonSelectPrintFile
// ABSTRACT: Handler for select print file selection button being pressed
//******************************************************************************************

void CXpm_Pmpd_DlgMain::OnBnClickedButtonSelectPrintFile()
{
   BOOL fResult = FALSE;                  // Result of this function
   OPENFILENAME  sOpenFileName;           // OpenFileName structure
   
   //-- Initialize OPENFILENAME structure
   ZeroMemory(&sOpenFileName, sizeof(sOpenFileName));
   sOpenFileName.lStructSize     = sizeof(sOpenFileName);
   sOpenFileName.hwndOwner       = this->GetSafeHwnd();
   sOpenFileName.lpstrTitle      = "XPM PRINT MONITOR PLUGIN DEMO - SELECT PRINT FILE";
   sOpenFileName.lpstrFile       = m_szPrintFile;
   sOpenFileName.nMaxFile        = sizeof(m_szPrintFile);
   sOpenFileName.lpstrFilter     = "PRN (*.prn)\0*.PRN\0BIN (*.bin)\0*.BIN\0All(*.*)\0*.*\0";
   sOpenFileName.nFilterIndex    = 1;
   sOpenFileName.lpstrFileTitle  = NULL;
   sOpenFileName.nMaxFileTitle   = 0;
   sOpenFileName.lpstrInitialDir = NULL;
   sOpenFileName.Flags = OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST;

   //-- Display the open/saev file name dialog box. 
   if (GetOpenFileNameA(&sOpenFileName)) {

      char szDrive[_MAX_DRIVE], szDir[_MAX_DIR], szFname[_MAX_FNAME], szExt[_MAX_EXT], szFile[MAX_PATH];
      
      //-- Check if file exsists
      if (PathFileExists(m_szPrintFile)) {

         CString strTxt;

         //-- Split path and set extension if no extension set
         _splitpath(m_szPrintFile, szDrive, szDir, szFname, szExt);
         _makepath(szFile, "", "", szFname, szExt);
 
         //-- Setup print file
         strTxt.Format("PRINT FILE => \"%s\"", szFile);
         GetDlgItem(IDC_BUTTON_SELECT_PRINT_FILE)->SetWindowText(strTxt);
      }
      else GetDlgItem(IDC_BUTTON_SELECT_PRINT_FILE)->SetWindowText("SELECT PRINT FILE");
   }
}
