/***********************************************************************\
**
**                          Copyright (c) 2008-2022 by
**                          HENGSTLER GmbH, Aldingen
**
**      This  software  is furnished under a license and may be used and
**      copied only in accordance with the terms  of  such  license  and
**      with the inclusion of the above copyright notice.  This software
**      or any other copies thereof may not  be  provided  or  otherwise
**      made  available  to any other person.  No title to and ownership
**      of the software is hereby transferred.
**
**      The  information  in  this software is subject to change without
**      notice and should not be construed as a commitment by Hengstler.
**
**  Module:      Xpm_Pmp_PrinterFeed.cpp
**  Facility:    S690001 - Hengstler XPM Printer Driver for Windows XP/7/8/10
**  Component:   Print Monitor Plugin (Xpm-PrintMonPlugin)
**  Version:     R8-V1.07-FINAL / 2022OCT18
**  Environment: Windows XP/7/8/10
**  Compiler:    Microsoft Visual Studio 2015
**  Abstract:    Module that manages the printer feed status structure entry in the registry
**  Note:        -
**  History:     R1-V1.00c / 2008JUL11 / HOR / 1st RELEASE - 1st BETA
**               R1-V1.00d / 2008DEC11 / HOR / 1st RELEASE - 2nd BETA
**               R1-V1.00e / 2009MAY05 / HOR / 1st RELEASE - 3rd BETA
**               R1-V1.00f / 2009JUL16 / HOR / 1st RELEASE - 4th BETA
**               R1-V1.00  / 2009AUG27 / HOR / 1st RELEASE - FINAL 
**               ------------------------------------------------------------
**               R2-V1.01a / 2010JAN26 / HOR / 2nd RELEASE - 1st BETA 
**               R2-V1.01b / 2010MAR04 / HOR / 2nd RELEASE - 2nd BETA
**               R2-V1.01c / 2011DEC19 / HOR / M-R2-012-HOR
**                  Complete driver package converted to VS2010 and adapted to Windows 7 32/64 bit
**               R2-V1.01c / 2011DEC19 / HOR / 2nd RELEASE - 3rd BETA
**               R2-V1.01d / 2012JUN27 / HOR / 2nd RELEASE - 4th BETA
**               R2-V1.01e / 2015MAY05 / HOR / 2nd RELEASE - 5th BETA
**               R2-V1.01f / 2016FEB15 / HOR / 2nd RELEASE - 6th BETA
**               R2-V1.01  / 2016AUG19 / HOR / M-R2-041-HOR
**                 Complete driver package converted to VS2015 and adapted to Windows 8/10 32/64 bit
**               R2-V1.01  / 2016AUG19 / HOR / 2nd RELEASE - FINAL
**               ------------------------------------------------------------
**               R3-V1.02  / 2016AUG24 / HOR / M-R3-001-HOR
**                 Additional driver package compilation added for usage within Windows-XP.
**               R3-V1.02  / 2016AUG24 / HOR / 3rd RELEASE - FINAL
**               ------------------------------------------------------------
**               R4-V1.03  / 2018JUL10 / HOR / 4th RELEASE - FINAL
**               ------------------------------------------------------------
**               R5-V1.04  / 2018OCT11 / HOR / 5th RELEASE - FINAL
**               ------------------------------------------------------------
**               R6-V1.05  / 2018DEC07 / HOR / 6th RELEASE - FINAL
**               ------------------------------------------------------------
**               R7-V1.06  / 2022FEB10 / HOR / 7th RELEASE - FINAL
**               ------------------------------------------------------------
**               R8-V1.07  / 2022OCT18 / HOR / 8th RELEASE - FINAL
**               ------------------------------------------------------------
**
\**********************************************************************/

/*
** INCLUDES
*/

#include "Xpm_Pmp_System.h"
#include "Xpm_PrintMonPlugin.h"
#include "Xpm_Pmp_Interface.h"
#include "Xpm_Pmp_RegistryAccess.h"
#include "Xpm_Pmp_PrinterFeed.h"

/*
** COMPILER 
*/

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

/*
** MACROS
*/

/*
** TYPES
*/

/*
** FUNCTIONS
*/

/*
** VARIABLES
*/

//******************************************************************************************
// FUNCTION   Xpm_Pmp_Pfe_InitializePrinterFeed
// ABSTRACT   Initializes printer feed status
//******************************************************************************************

bool Xpm_Pmp_Pfe_InitializePrinterFeed
   (P_XPM_PLUGIN_DATA psData)
{
   bool                      fResult           = false;  // Return value, FALSE by default
   TS_XPM_PMP_PRINTER_FEED sPrinterFeed;             // Printer status data structure (registry)

   //-- Validate parameter
   if (!psData || (psData->dwSructId != P_XPM_PLUGIN_DATA_ID))
      goto FAILURE;

   //-- Initialize printer inquiry data structure 
   memset(&sPrinterFeed, 0, sizeof(sPrinterFeed));         // Clear complete structure
   sPrinterFeed.dwSize          = sizeof(sPrinterFeed);    // Set size of this structure
   sPrinterFeed.ubRevision      = D_XPM_PMP_PFE_REVISION;  // Set revision number (D_XPM_PMP_PFE_REVISION)
   sPrinterFeed.dwTsLastUpdate  = 0;                       // Set timestamp of last update
   sPrinterFeed.dwDataValidFlag = 0;                       // Reset data valid flags (see D_XPM_PMP_PFE_DVF_...)

   //-- Set time stamp
   sPrinterFeed.dwTsLastUpdate = GetTickCount();

   //-- Write printer feed status to registry
   if (!Xpm_Pmp_Ra_WritePrinterDataToRegistry(psData, D_XPM_PMP_REG_VALUE_PMP_PRINTER_FEED, (PBYTE) &sPrinterFeed, sizeof(sPrinterFeed)))
      goto FAILURE;

   //-- Success
   fResult = true;

   //-- All done
   goto EXIT;

FAILURE:
   ;

EXIT:

   //-- Return TRUE on success and FALSE otherwise
   return(fResult);
}

//******************************************************************************************
// FUNCTION   Xpm_Pmp_Pfe_UpdatePrinterFeed
// ABSTRACT   Updates the printer feed status 
//******************************************************************************************

bool Xpm_Pmp_Pfe_UpdatePrinterFeed
   (P_XPM_PLUGIN_DATA psData, bool fPrinterMightBeBusy, bool fInvalidateOnly)
{
   bool                            fResult                    = false;                   // Return value, FALSE by default
   TS_XPM_PMP_PRINTER_FEED         sPrinterFeed;                                        // Printer status data structure (registry)
   BYTE                            aubEcPrinterFeed[]       = {0x1d,0x61,0x00,0x10};    // GS a+[n1]+[n2],n=0x0010 => emulation command for requesting printer feed status
   BYTE                            aubPrinterFeed[sizeof(TS_XPM_API_PCKT_HEADER) + sizeof(TS_XPM_API_PCKT_PRINTER_FEED)]; 
                                                                                         // Printer feed status bytes
   DWORD                           dwReadTimeOutMs;                                      // Read timeout in ms
   
   //-- Validate parameter
   if (!psData || (psData->dwSructId != P_XPM_PLUGIN_DATA_ID))
      goto FAILURE;

   //-- Initialize printer feed status structure 
   memset(&sPrinterFeed, 0, sizeof(sPrinterFeed));        // Clear complete structure
   sPrinterFeed.dwSize          = sizeof(sPrinterFeed);   // Set size of this structure
   sPrinterFeed.ubRevision      = D_XPM_PMP_PFE_REVISION; // Set revision number (D_XPM_PMP_PFE_REVISION)
   sPrinterFeed.dwTsLastUpdate  = 0;                      // Set timestamp of last update
   sPrinterFeed.dwDataValidFlag = 0;                      // Reset data valid flags (see D_XPM_PMP_PFE_DVF_...)

   //-- Check if data must be requested from the printer or if data is to be invalidated
   if (fInvalidateOnly) 
      goto DONE;

   //-- Inquire printer feed status 
   {
      //-- Inquire printer feed status 
      //.. Set read timeout printer response to inquiry to .. ms, depending if printer could be busy) 
      dwReadTimeOutMs = (!fPrinterMightBeBusy ? 300 : 15000);
      if (!Xpm_Pmp_Ifc_InquirePrinterData(psData, D_XPM_API_PID_TX_PRINTER_FEED, aubEcPrinterFeed, sizeof(aubEcPrinterFeed), aubPrinterFeed, sizeof(aubPrinterFeed), dwReadTimeOutMs)) 
         goto DONE;

      //-- Copy printer feed status to storage
      memcpy(&sPrinterFeed.sPrinterFeed, &aubPrinterFeed[sizeof(TS_XPM_API_PCKT_HEADER)], sizeof(sPrinterFeed.sPrinterFeed));

      //-- Set printer feed status valid flag
      sPrinterFeed.dwDataValidFlag |= D_XPM_PMP_PFE_DVF_PRINTER_FEED;
   }

   //-- Success (inquiry succesfull)
   fResult = true;

DONE:

   //-- Set time stamp
   sPrinterFeed.dwTsLastUpdate = GetTickCount();

   //-- Write printer feed status to registry
   if (!Xpm_Pmp_Ra_WritePrinterDataToRegistry(psData, D_XPM_PMP_REG_VALUE_PMP_PRINTER_FEED, (PBYTE) &sPrinterFeed, sizeof(sPrinterFeed)))
      goto FAILURE;

   //-- All done
   goto EXIT;

FAILURE:
   ;

EXIT:

   //-- Return TRUE on success and FALSE otherwise
   return(fResult);
}
